#!/usr/bin/env python

# 
# Copyright (C) 2010 Simon Pickering <S.G.Pickering@bath.ac.uk>
#
#
#
# GNU General Public Licence (GPL)
# 
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 59 Temple
# Place, Suite 330, Boston, MA  02111-1307  USA
#

import sys
import re, string
import os

if os.name<>'nt':
    import dbus
    import dbus.service
    import dbus.mainloop.qt

from PyQt4 import QtCore, QtGui, QtSql
#import hildon

#import application_rc

#self.setAttribute(Qt.WA_Maemo5StackedWindow, True)

# ==============================================================================

class itemDB():
    def __init__(self):
        QtGui.QMainWindow.__init__(self)
        
        if os.name<>'nt':
            os.environ[DBUS_STARTER_ADDRESS]
            self.session_bus = dbus.SessionBus()
            self.proxy = bus.get_object('org.maemo.itemdb',
                       '/org/maemo/itemdb')

      
# ==============================================================================  
#'posix', 'nt', 'os2', 'ce', 'java', 'riscos'  
if os.name=='posix':
    class itemDB_DBus(dbus.service.Object):
        #http://dbus.freedesktop.org/doc/dbus-python/doc/tutorial.html#data-types
        @dbus.service.method("org.maemo.mbarcode", in_signature='ss', out_signature='b')
        def HandleBarcode(self, barcodeType, barcodeData):
            print barcodeType
            print barcodeData
			
			return True
        
# ==============================================================================    

class Database():
    
	tableListGenString = ''' (key PRIMARY KEY, 
						sqlTableName VARCHAR(255), 
						prettyTableName VARCHAR(255),
						) '''
	
    tableGenString = ''' (key PRIMARY KEY, 
                    ean VARCHAR(13) NOT NULL, 
                    title VARCHAR(255), 
                    author VARCHAR(255), 
                    price FLOAT, 
                    quantity FLOAT, 
                    location VARCHAR(255), 
                    timedate TIMESTAMP, 
                    thumbnail VARBINARY, 
                    review TEXT, 
                    other TEXT, 
                    itemType TINYINT)'''

    def __init__(self):
        # this is so we can alter the labels depending on the contents of the particular db entry
        self.itemTypeDict = dict(unknown=0, book=1, CD=2, DVD=3)
        
        # just to establish some normal row types
        self.ItemDict = dict(key=0, ean="", title="", author="", price="", quantity="", location="", timedate = "", thumbnail="", review="", other="", itemType=0)        
        self.TableDict  = dict(key=0, prettyTableName="", sqlTableName="")

        # work out the database path
        if os.name<>'nt':
            dbpath = os.path.expanduser('~/.itemdb/')
        else:
            dbpath = os.path.expanduser('~/itemdb/')
        dbname = dbpath + 'itemdb.db'

        ## see if the db exists first of all
        _openDB(dbname)
        
        # see if the tables exist
        if not _checkTablesExist():
            # if not, create them
            _createTableListTable()
        
    def _createTableListTable():
        query = QtSql.QSqlQuery()
        query.exec_("""CREATE TABLE tablelist (
				key PRIMARY KEY,
				sqlname VARCHAR(255) UNIQUE NOT NULL,
				prettyname VARCHAR(255) NOT NULL)""")

    def _openDB(self, filename):
        self.db = QtSql.QSqlDatabase.addDatabase("QMYSQL")
        self.db.setHostName("localhost")
        self.db.setDatabaseName(filename)
        #self.db.setUserName("user")
        #self.db.setPassword("")
        return self.db.open()

    def _checkTablesExist():
        query = QtSql.QSqlQuery()
        # check that the tablelist table exists
        sql = 'SELECT key FROM tablelist LIMIT 1;'
        ret = query.exec_(sql)
        # returns true or false depending on whether the transaction succeeded
        return ret
        
    def closeDB(self):
       self.db.close()
    
    def writeValues(self, sql):
       self.cursor = self.db.cursor()
       self.cursor.execute(sql)
       self.cursor.fetchone()
       
# ==============================================================================
       
    def CreateTable(self, prettyTableName):
        # create a table in the database
        query = QtSql.QSqlQuery()
        
        # generate a valid sqlname
        sqlTableName = GenerateSqlName(prettyTableName)
        
        # send the SQL command to create the table
        sql = 'CREATE TABLE' + sqlTableName + tableGenString
        ret = query.exec_(sql)
        
        # add it to the tablelist table
        sql = "INSERT INTO tablelist (sqlTableName, prettyTableName) VALUES ('%s', '%s');" % (sqlTableName, prettyTableName)
        ret = query.exec_(sql)
        
        # return a model object??
		return GetTableModel(sqlTableName)
       
    def GetTableListModel(self):
        # return the tablelist table model 
        model = QtSql.QSqlTableModel()
        model.setTable('tablelist')
        return model
        
    def GetTableModel(self, sqlTableName):
        # return the table model
        model = QtSql.QSqlTableModel()
        model.setTable(sqlTableName)
        return model
        
    def PrettyToSqlName(self, prettyName):
        # lookup the sql name in the tablelist table and return the pretty name
        query = QtSql.QSqlQuery(self.db)
        query.exec_("SELECT * FROM tablelist WHERE prettyname='" + prettyName +"'")
        while query.next():
            return query.value(self.TableDict['sqlTableName']).toString()

    def SqlToPrettyName(self, sqlName):
        # lookup the sql name in the tablelist table and return the pretty name
        query = QtSql.QSqlQuery(self.db)
        query.exec_("SELECT * FROM tablelist WHERE sqlname='" + sqlName +"'")
        while query.next():
            return query.value(self.TableDict['prettyTableName']).toString()

    def GenerateSqlName(self, prettyName):
        # generate a valid sql name based on the pretty name
        # make sure it does not appear in the tablelist already!
        
        # lookup the pretty name in the tablelist table and return the sql name
        if prettyName.isalnum():
            return prettyName
        
        # http://stackoverflow.com/questions/1276764/stripping-everything-but-alphanumeric-chars-from-a-string-in-python
        delchars = ''.join(c for c in map(chr, range(256)) if not c.isalnum())
        sqlName = prettyName.translate(None, delchars)
        
        # now see if this name already exists in the db
        currentNumber = 0
        while 1:
            if CheckTableExists(sqlName + str(currentNumber))==False:
                return sqlName + str(currentNumber)
            else:
                # if it does, tack on some numbers, if not just return it
                currentNumber = currentNumber + 1

    def GetSelectedTableModel(self):
        # see which table is selected in the list/tree-view
        pass
        
    def RemoveTable(self, sqlName):
        sqlName = GetSelectedTableModel()
        
        query = QtSql.QSqlQuery(self.db)
        
        # remove from the tablelist
        query.exec_("DELETE * FROM tablelist WHERE sqlname='" + sqlName +"'")
        
        # remove the table itself
        query.exec_("DROP TABLE '" + sqlName + "'")
        
    def CheckTableExists(self, sqlName):
        query = QtSql.QSqlQuery()
        # check that the tablelist table exists
        sql = "SELECT key FROM " + sqlName + " LIMIT 1;"
        ret = query.exec_(sql)
        # returns true or false depending on whether the transaction succeeded
        #query.finish() # apparently not needed
        return ret

    def AddItem(self, sqlName, itemDict):
        # add an item to the table in question
        query = QtSql.QSqlQuery()
		# not sure about the datetime type nor the thumbnail type!!!!
        sql = "INSERT INTO " + sqlName + " (\
												ean, title, author, price, \
												quantity, location, timedate, thumbnail, \
												review, other, itemType \
											) VALUES (\
												'%s', '%s', '%s', '%f', \
												'%f', '%s', '%i', '%s', \
												'%s', '%s', '%i' \
											);" % (itemDict)
		ret = query.exec_(sql)
    
    def RemoveItem(self, sqlName, key):
        # remove an item from the table in question
        query = QtSql.QSqlQuery(self.db)
        # remove from the tablelist
        query.exec_("DELETE * FROM " + sqlName + " WHERE key=" + key)
 
    def ChangeItem(self, sqlTableName, itemDict):
        # change an item in the table in question
        pass
        #    self.sql = 'INSERT INTO jobs (incident_id, organization, organization_poc, media_type) VALUES ("%s", "%s", "%s", "%s")' % (self.edi_IncidentID.text(), self.edi_OrganizationAffected.text(), self.edi_OrganizationContact.text(), self.edi_MediaType.text())

    def SearchForItem(self):
        # search the various tables and return anything that matches
        pass

    def GetSelectedItem(self):
        pass
    
    
# ==============================================================================    
    
    
class SummaryWindow(QtGui.QWidget):
    def __init__(self, db, parent=None):
        QtGui.QWidget.__init__(self, parent)
        
        self.db = db
        
        self.btnSave = QtGui.QPushButton('Save')
        self.btnSave.setEnabled(0)
        self.connect(self.btnSave, QtCore.SIGNAL('clicked()'), self.doSaveBarcode)
        self.btnViewReviews = QtGui.QPushButton("View Reviews")
        self.btnViewReviews.setEnabled(0)
        self.connect(self.btnViewReviews, QtCore.SIGNAL('clicked()'), self.doViewReviews)
        self.btnTypeBarcode = QtGui.QPushButton("Type in barcode")
        self.btnTypeBarcode.setEnabled(1)
        self.connect(self.btnTypeBarcode, QtCore.SIGNAL('clicked()'), self.doTypeBarcode)
        
        self.buttonLayout = QtGui.QVBoxLayout()
        self.buttonLayout.addWidget(self.btnSave)
        self.buttonLayout.addWidget(self.btnViewReviews)
        self.buttonLayout.addWidget(self.btnTypeBarcode)
        

        self.descriptionLabel = QtGui.QLabel()
        self.searchResultsList = QtGui.QTreeView()
        self.listLayout = QtGui.QVBoxLayout()
        self.listLayout.addWidget(self.descriptionLabel)
        self.listLayout.addWidget(self.searchResultsList)
        
        self.overallLayout = QtGui.QHBoxLayout() # overall container layout
        self.overallLayout.addLayout(self.listLayout) # add lists container 
        self.overallLayout.addLayout(self.buttonLayout) # add button container
        


        # need to create some add item dialog windows too


        
        # add layout to the container widget
        self.setLayout(self.overallLayout)


    def doTypeBarcode(self):
        # open dialog and allow user to type or c&p in barcode data
        (barcode, okPressed) = QtGui.QInputDialog.getText(self, "Input barcode", "Barcode:", QtGui.QLineEdit.Normal, "")
        if okPressed:
            setBarcode(barcode)

    def doSaveBarcode(self):
        # open dialog and allow user to save this barcode data
        # allow user to edit data and choose which table to save the data to
        pass
    
    
    def doViewReviews(self):
        # open a dialog/sub-window and show reviews of the item in question
        pass


    def setBarcode(self, barcode):
        self.barcode = barcode
        self.lookup = LookupService(self.barcode)
        
        if self.lookup.isBook:
            text = self.lookup.getEan + chr(13) +
                   self.lookup.getISBN + chr(13) +
                   self.lookup.getName + chr(13) +
                   self.lookup.getTitle + chr(13) +
                   self.lookup.getPrice
        else if self.lookup.isDVD:
            text = self.lookup.getEan + chr(13) +
                   self.lookup.getName + chr(13) +
                   self.lookup.getTitle + chr(13) +
                   self.lookup.getPrice
       else if self.lookup.isCD:
            text = self.lookup.getEan + chr(13) +
                   self.lookup.getName + chr(13) +
                   self.lookup.getTitle + chr(13) +
                   self.lookup.getPrice
        else:
            text = self.lookup.getEan + chr(13) +
                   self.lookup.getDescription + chr(13) +
                   self.lookup.getManufacturer + chr(13) +
                   self.lookup.getPrice

        self.descriptionLabel = text




        if self.lookup.isBook:
            self.lookup.getEan
            self.lookup.getISBN
            self.lookup.getName
            self.lookup.getTitle
            
        else if self.lookup.isDVD:
            self.lookup.getEan
            self.lookup.getName
            self.lookup.getTitle
            
       else if self.lookup.isCD:
            self.lookup.getEan
            self.lookup.getName
            self.lookup.getTitle
            
        else:
            self.lookup.getEan
            self.lookup.getDescription
            self.lookup.getManufacturer
            


        SearchForItem

        
        
        # now see if we already have this item in the database
        
        self.searchResultsList = text
        

# ==============================================================================

class LookupService():
    def __init__(self, barcode, parent=None):
        
    def isBook:
        
    def isDVD:

    def isCD:

    def getEan:
        
    def getISBN:
        
    def getTitle:
        
    def getName:
        
    def getManufacturer:
        
    def getDescription:
        
    def getPrice:


# ==============================================================================

    
class TableListWindow(QtGui.QWidget):
    def __init__(self, db, sqlName, parent=None):
        QtGui.QWidget.__init__(self, parent)
        
        self.tableModel = QtSql.QSqlTableModel()
        self.tableModel.setTable(sqlName)
        self.tableModel.setEditStrategy(QtSql.QSqlTableModel.OnRowChange)
        self.tableModel.select()
        
        #self.tableModel.removeColumn(0); # don't show the ID
        #self.tableModel.setHeaderData(0, Qt.Horizontal, "Name");
        #self.tableModel.setHeaderData(1, Qt.Horizontal, "Salary");
        
        
        self.tableList = QtGui.QTableView()
        self.tableList.setModel(self.tableModel)
        

        
        #query = QtSql.QSqlQuery(db)

        #self.tableList.setModel()
        

        self.btnAddTable = QtGui.QPushButton('Add Table')
        self.btnAddTable.setEnabled(1)
        self.connect(self.btnAddTable, QtCore.SIGNAL('clicked()'), self.doAddTable)
        self.btnRenameTable = QtGui.QPushButton("Rename Table")
        self.btnRenameTable.setEnabled(0)
        self.connect(self.btnRenameTable, QtCore.SIGNAL('clicked()'), self.doRenameTable)
        self.btnDeleteTable = QtGui.QPushButton("Delete Table")
        self.btnDeleteTable.setEnabled(0)
        self.connect(self.btnDeleteTable, QtCore.SIGNAL('clicked()'), self.doDeleteTable)
        
        self.buttonLayout = QtGui.QVBoxLayout()
        self.buttonLayout.addWidget(self.btnAddTable)
        self.buttonLayout.addWidget(self.btnRenameTable)
        self.buttonLayout.addWidget(self.btnDeleteTable)
        
        
        
        self.overallLayout = QtGui.QHBoxLayout() # overall container layout
        self.overallLayout.addWidget(self.tableList) # add lists container 
        self.overallLayout.addLayout(self.buttonLayout) # add button container


        # add layout to the container widget
        self.setLayout(self.overallLayout)

    def doAddTable(self):
        (prettyName, okPressed) = QtGui.QInputDialog.getText(self, "Create Table", "Table name:", QtGui.QLineEdit.Normal, "")
        # check to see if this pretty name already exists, not much point in having two with the same name....
        
        # we could automatically add a number to the end I suppose
        
        # do the SQL stuff to add this table
        
        
        
    def doRenameTable(self):
        # find the current table name
        (prettyName, sqlName) = getSelectedTable()
        (prettyName, okPressed) = QtGui.QInputDialog.getText(self, "Rename Table", "New table name:", QtGui.QLineEdit.Normal, prettyName)
        # check to see if this pretty name already exists, not much point in having two with the same name....
        
        # we could automatically add a number to the end I suppose
        
        # do the SQL stuff to add this table
        
    def doDeleteTable(self):
        # find the current table name
        (prettyName, sqlName) = getSelectedTable()
        reply = QtGui.QMessageBox.question(self, 'Delete Table',
            "Delete table" + prettyName + "?", QtGui.QMessageBox.Yes, QtGui.QMessageBox.No)
        
        if reply == QtGui.QMessageBox.Yes:
            # do SQL stuff to remove the table
            pass

    def getSelectedTable(self):
        # return the selected table names (pretty and sql)
        pass




# ==============================================================================


class TableContentsWindow(QtGui.QWidget):
    def __init__(self, db, sqlName, parent=None):
        QtGui.QWidget.__init__(self, parent)

        self.model = QtSql.QSqlTableModel()
        self.model.setTable(sqlName)
        self.model.setEditStrategy(QtSql.QSqlTableModel.OnRowChange)
        self.model.select()
        
        #self.model.removeColumn(0); # don't show the ID
        #self.model.setHeaderData(0, Qt.Horizontal, "Name");
        #self.model.setHeaderData(1, Qt.Horizontal, "Salary");
        
        
        self.view = QtGui.QTableView()
        self.view.setModel(self.model)
        

        #self.btnEditItem = QtGui.QPushButton('Edit Item')
        #self.btnEditItem.setEnabled(0)
        #self.connect(self.btnEditItem, QtCore.SIGNAL('clicked()'), self.doEditItem)
        self.btnCopyItem = QtGui.QPushButton("Copy Item")
        self.btnCopyItem.setEnabled(0)
        self.connect(self.btnCopyItem, QtCore.SIGNAL('clicked()'), self.doCopyItem)
        self.btnMoveItem = QtGui.QPushButton("Move Item")
        self.btnMoveItem.setEnabled(0)
        self.connect(self.btnMoveItem, QtCore.SIGNAL('clicked()'), self.doMoveItem)
        self.btnDeleteItem = QtGui.QPushButton("Delete Item")
        self.btnDeleteItem.setEnabled(0)
        self.connect(self.btnDeleteItem, QtCore.SIGNAL('clicked()'), self.doDeleteItem)
        
        self.buttonLayout = QtGui.QVBoxLayout()
        #self.buttonLayout.addWidget(self.btnEditItem)
        self.buttonLayout.addWidget(self.btnCopyItem)
        self.buttonLayout.addWidget(self.btnMoveItem)
        self.buttonLayout.addWidget(self.btnDeleteItem)
        
        
        
        self.overallLayout = QtGui.QHBoxLayout() # overall container layout
        self.overallLayout.addWidget(self.view) # add lists container 
        self.overallLayout.addLayout(self.buttonLayout) # add button container


        # add layout to the container widget
        self.setLayout(self.overallLayout)
    
    def ChangeCurrentTable(self, prettyTable, sqlTable):
        # call this to get the view to update its contents if the selected table changes
        pass
    
#    def doEditItem(self):
#        pass
        
    def doCopyItem(self):
        pass
        
    def doMoveItem(self):
        pass  
  
    def doDeleteItem(self):
        pass


# ==============================================================================
    
class MainWindow(QtGui.QMainWindow):
    def __init__(self):
        QtGui.QMainWindow.__init__(self)

        self.tabWidget = QtGui.QTabWidget()
        #self.tabWidget.resize(250, 150)
        self.tabWidget.setWindowTitle('ItemDB')
        #self.tabWidget.setWindowIcon(QtGui.QIcon('icons/web.png'))
        
        self.db = Database()
        
        if os.name<>'nt':
            os.environ[DBUS_STARTER_ADDRESS]
            self.session_bus = dbus.SessionBus()
            self.proxy = bus.get_object('org.maemo.itemdb',
                       '/org/maemo/itemdb')        
        
        # =========================================
        # add ui elements
        # =========================================
        
        # -----------------------------------------
        # summary page - description of item, results of search of db, buttons to add to list/move/etc
        # -----------------------------------------
        

        self.summaryWindow = SummaryWindow(self.db)


        # -----------------------------------------
        # table list - list of tables, new, rename, delete buttons
        # -----------------------------------------
        sqlName=''
        self.tableListWindow = TableListWindow(self.db, sqlName)
        

        # -----------------------------------------
        # table contents - list of items, delete, edit buttons
        # -----------------------------------------
        
        self.tableContentsWindow = TableContentsWindow(self.db, "") # this will be a subwindow of the above


        # -----------------------------------------
        # Add all the tabs to the main window
        # -----------------------------------------
        
        self.tabWidget.insertTab (0, self.summaryWindow, "Summary")
        self.tabWidget.insertTab (1, self.tableListWindow, "Table List")
        self.tabWidget.insertTab (2, self.tableContentsWindow, "Table Contents")
        
        
        
        self.setCentralWidget(self.tabWidget)
        #self.tabWidget.resize(250, 150)
        self.tabWidget.setWindowTitle('ItemDB')
        self.tabWidget.show()        
        
        
        self.readSettings()

        #self.connect(self.textEdit.document(), QtCore.SIGNAL("contentsChanged()"),
        #             self.documentWasModified)
        
        

        

    

    def closeEvent(self, event):
        self.writeSettings()
        event.accept()


    def about(self):
        QtGui.QMessageBox.about(self, self.tr("About Application"),
            self.tr("<b>ItemDB</b> "))


    def readSettings(self):
        settings = QtCore.QSettings("maemo.org", "ItemDB")
        pos = settings.value("pos", QtCore.QVariant(QtCore.QPoint(200, 200))).toPoint()
        size = settings.value("size", QtCore.QVariant(QtCore.QSize(400, 400))).toSize()
        self.resize(size)
        self.move(pos)

    def writeSettings(self):
        settings = QtCore.QSettings("maemo.org", "ItemDB")
        settings.setValue("pos", QtCore.QVariant(self.pos()))
        settings.setValue("size", QtCore.QVariant(self.size()))


# ==============================================================================


if __name__ == "__main__":
    if os.name<>'nt':
        dbus.mainloop.qt.DBusQtMainLoop(set_as_default=True) 
    
    app = QtGui.QApplication(sys.argv)
    mainWin = MainWindow()
    mainWin.show()

    

    sys.exit(app.exec_())
